/*=====================================================================
  File:      MCLangType.cs

  Summary:   Brief summary of the file contents and purpose.

---------------------------------------------------------------------
  This file is part of the Microsoft NGWS SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

namespace LangUtil {

using System;
using System.Text;
using System.Reflection;
using System.Collections;

   public class MCLangType : LangType
   {
      // constructor calls base constructor, passing Type object, then further processing
      public MCLangType( Type ThisType ) : base( ThisType )
      {      
      }
   
      // Constructor calls base constructor, passing Type object, then further processing
      public MCLangType( ParameterInfo ParamInfo ) : base( ParamInfo )
      {      
      }
      
      override public string TranslateTypeName( string strName )
      {
         // Strip the brackets here, if any
         bool bBrackets = false;
         if ( strName.Substring( strName.Length - 2 ) == "[]" )
         {
            bBrackets = true;
            strName = strName.Substring( 0, strName.Length - 2 );
         }

         // Translate the class name
         if ( strName == "Void" )
            strName = "void";
         else if ( strName == "Int16" )
            strName = "short";
         else if ( strName == "Int32" )
            strName = "int";
         else if ( strName == "Int64" )
            strName = "__int64";
         else if ( strName == "UInt16" )
            strName = "unsigned short";
         else if ( strName == "UInt32" )
            strName = "unsigned int";
         else if ( strName == "UInt64" )
            strName = "unsigned __int64";
         else if ( strName == "Single" )
            strName = "float";
         else if ( strName == "Double" )
            strName = "double";
         else if ( strName == "Boolean" )
            strName = "bool";
         else if ( strName == "Char" )
            strName = "wchar_t";
         else if ( strName == "Byte" )
            strName = "unsigned char";
         else if ( strName == "SByte" )
            strName = "char";

         // Finally, append the brackets, if needed
         if ( bBrackets )
            strName += "[]";

         //=================================================================//
         // append * for nonvalue types.
         //
         //
         // if (!(m_TypeObject.IsValueType || (m_TypeObject.IsArray && m_TypeObject.GetArrayElementType().IsValueType)))
         // name = name + "*"; 
         //
         //
         //=================================================================//
         
         // confirm this is basically redundant with IsPointer type, which will, I take it, be true 
         // for all reference types. 
         // Now the question is in what order to reference marks and pointers
         // get appended to C++ parameters and return types.

         if ( !m_IsByRef )
         {
            if ( !( m_TypeObject.IsValueType || ( m_TypeObject == System.Type.GetType( "System.Void" ) ) ||
                  ( ( m_TypeObject.IsArray && ( m_TypeObject != System.Type.GetType( "System.Array" ) ) ) && 
                    ( m_TypeObject.GetElementType().IsValueType || m_TypeObject == System.Type.GetType( "System.Void" ) ) ) ) )
               strName = strName + "*";
         }

         return strName;
      }

      public string HTMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string XMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string Access
      {
         get
         {
            if ( m_TypeObject.IsPublic )
               return "public: ";
            else
               return String.Empty;
         }
      }

      public string Attributes
      {
         get
         {
            StringBuilder s = new StringBuilder( m_ATTRIBUTE_STRING_SIZE );
         
            if ( m_TypeObject.IsAbstract && !m_TypeObject.IsInterface )
            {
               s.Append( "__abstract " );
            }
            else if ( m_TypeObject.IsSealed && !m_TypeObject.IsValueType && !m_TypeObject.IsSubclassOf( typeof( Delegate ) ) &&
                      !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               s.Append( "__sealed " );
            }

            return s.ToString();
         }   
      }

      public string TypeCategory
      {
         get
         {
            string s = String.Empty;
            if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && 
                 !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               StringBuilder sb = new StringBuilder( "__delegate " );
               MethodInfo m = m_TypeObject.GetMethod( "Invoke" );

               if ( m.ReturnType.Name.Equals( "Void" ) )
                  sb.Append( "void" );
               else
                  sb.Append( TranslateTypeName( m.ReturnType.Name ) );

               sb.Append( " " );
               sb.Append( m_TypeObject.Name );

               ParameterInfo[] p = m.GetParameters();
               Type[] param_types = new Type[p.Length];
               for ( int i = 0; i < param_types.Length; i++ )
                  param_types[i] = p[i].ParameterType;

               sb.Append( "(" );
               for ( int i = 0; i < p.Length; i++ )
               {
                  sb.Append( String.Format( "{0} {1}", TranslateTypeName( param_types[i].Name ), p[i].Name ) );
                  if ( i + 1 < p.Length )
                     sb.Append( ", " );
               }
            
               sb.Append( ")" );
               s = sb.ToString();
            }        
            else if ( m_TypeObject.IsInterface )
               s = "__interface";
            else if ( m_TypeObject.IsEnum )
               s = "__value enum";
            else if ( m_TypeObject.IsValueType )
               s = "__value struct";
            else 
               s = "class";
            return s;
         }
      }


      public string Extends
      {
         get
         {
              try{
                    if (m_TypeObject.IsUnmanagedValueType)
                       return String.Empty;
                    if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && 
                         !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
                       return String.Empty;
                    if ( m_TypeObject.IsSubclassOf( typeof( Enum ) ) && !( m_TypeObject == typeof( Enum ) ) )
                       return String.Empty;
                    StringBuilder s = new StringBuilder();
                     
                    bool found = false;

                    // There is only a single parent class, if we are doing Object, then it has no parent,
                    // otherwise, if the parent is just object, we can ignore that, it is assumed.
                    if ( m_TypeObject != typeof( Object ) && 
                         m_TypeObject.BaseType != typeof( Object ) && 
                         !m_TypeObject.IsInterface )
                    {
                       found = true;
                       String temp = m_TypeObject.BaseType.Name;
                       s.Append( String.Format( " : {0}", temp ) );
                    }

                    // Now we print the interfaces... there could be any number of these
                    Hashtable hash = new Hashtable();
                    for ( Type cur = m_TypeObject.BaseType; cur != null; cur = cur.BaseType )
                    {
                       Type[] Interfaces = cur.GetInterfaces();
                       for ( int i = 0; i < Interfaces.Length; i++ )
                       {
                          if ( !hash.ContainsKey( Interfaces[i] ) ) 
                             hash.Add( Interfaces[i], null );
                       }
                    }
                    Type[] interfaces = m_TypeObject.GetInterfaces();
                    ArrayList list = new ArrayList();
                    for ( int i = 0; i < interfaces.Length; i++ )
                    {
                       if ( !hash.ContainsKey( interfaces[i] ) ) 
                          list.Add( interfaces[i] );
                    }
            
                    // interfaces = (Type[])list.ToArray( typeof( Type ) );
                    interfaces = new Type[list.Count];
		              for ( int listIndex = 0; listIndex < list.Count; listIndex++ )
                    {
			              interfaces[listIndex] = (Type)list[listIndex];
		              }

                    // Type[] interfaces = m_TypeObject.GetInterfaces();
                    if ( interfaces.Length != 0 )
                    {
                       String temp = interfaces[0].Name;
                       if ( !found ) 
                          s.Append( ": " );
                       else 
                          s.Append( ", " );
                       s.Append( temp );
                    }
                    for ( int i = 1; i < interfaces.Length; i++ )
                    {
                       String temp = interfaces[i].Name;
                       s.Append( String.Format( ", {0}", temp ) );
                    }
                    
                    return s.ToString();      
             }
             catch(Exception Ex){
                  throw new Exception("Problem at: " + Ex.TargetSite.Name, Ex);
             }
             

         }   
      }

      public string TextDeclaration
      {
         override get
         {
            string m_Extends = String.Empty;
            string m_TypeName = String.Empty;

            if ( !( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && 
                 !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) ) )
            {
               m_Extends = Extends + ";";
               m_TypeName = " " + m_TypeObject.Name;
            }
            else
               m_TypeName += ";";
            return Access + Attributes + TypeCategory + m_TypeName + m_Extends;
         }
      }

      public MCConstructorInfo[] OrderedLangConstructors
      {
         get
         {
            ConstructorInfo[] Constructors = null;
            MCConstructorInfo[] MCConstructors = null;
            try
            {
               Constructors = m_TypeObject.GetConstructors( BindingFlags.NonPublic );
               MCConstructors = new MCConstructorInfo[Constructors.Length];
               ConstructorComparer ICompConst = new ConstructorComparer();
               Array.Sort( Constructors, ICompConst );
               for ( int i = 0; i < Constructors.Length; i++ )
               {
                  MCConstructors[i] = new MCConstructorInfo( Constructors[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in MCLangType.OrderedConstructors.", Ex );
            }
            return MCConstructors;
         }
      }

      public MCMethodInfo[] OrderedLangMethods
      {
         get
         {
            MethodInfo [] Methods = null;
            MCMethodInfo[] MCMethods = null;
            
            try
            {
               Methods = m_TypeObject.GetMethods( BindingFlags.NonPublic );
               MCMethods = new MCMethodInfo[Methods.Length];

               // Order the member categories
               MethodComparer ICompMet = new MethodComparer();
               Array.Sort( Methods, ICompMet ); 
               for ( int i = 0; i < Methods.Length; i++ )
               {
                  MCMethods[i] = new MCMethodInfo( Methods[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in MCLangType.OrderedConstructors.", Ex );
            }
            return MCMethods;
         }
      }

      public MCPropertyInfo[] OrderedLangProperties
      {
         get
         {
            int i = 0, j = 0;
            PropertyInfo[] Properties = null;
            MCPropertyInfo[] MCProperties = null;

            try
            {
               Properties = m_TypeObject.GetProperties( BindingFlags.NonPublic );
               MCProperties = new MCPropertyInfo[Properties.Length];
               PropertyComparer ICompProp = new PropertyComparer();
               Array.Sort( Properties, ICompProp );
               for ( ; i < Properties.Length; i++ )
               {
                  try
                  {
                     MCProperties[j] = new MCPropertyInfo( Properties[i] );
                     j++;
                  }
                  catch ( Exception /*ex*/ )
                  {                       
                     // Console.WriteLine( "MCProperty error {0}.{1}", Properties[i].ReflectedType.FullName, Properties[i].Name );
                  }
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in MCLangType.OrderedProperties", Ex );
            }

            MCPropertyInfo[] np = new MCPropertyInfo[j];
            for ( i = 0; i < j; i++ )
               np[i] = MCProperties[i];
            return np;
         }
      }

      public MCFieldInfo[] OrderedLangFields
      {
         get
         {
            FieldInfo[] Fields = null;
            MCFieldInfo[] MCFields = null;
            try
            {
               Fields = m_TypeObject.GetFields( BindingFlags.NonPublic );
               MCFields = new MCFieldInfo[Fields.Length];
               FieldComparer ICompField = new FieldComparer();
               Array.Sort( Fields, ICompField );
               for ( int i = 0; i < Fields.Length; i++ )
               {
                  MCFields[i] = new MCFieldInfo( Fields[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in MCLangType.OrderedFields.", Ex );
            }
            return MCFields;
         }
      }
   }

} // namespace LangUtil
