/*=====================================================================
  File:      VisualBasicLangType.cs

  Summary:   Brief summary of the file contents and purpose.

---------------------------------------------------------------------
  This file is part of the Microsoft NGWS SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

namespace LangUtil {

using System;
using System.Text;
using System.Reflection;
using System.Collections;

   public class VisualBasicLangType : LangType 
   {   
      // Constructors call base class constructor for both Types and ParameterInfos
      public VisualBasicLangType( Type ThisType ) : base( ThisType )
      {
      }

      public VisualBasicLangType( ParameterInfo ParamInfo ) : base( ParamInfo )
      {
      }

      // Overrides the base class constructor to provide the VB version of these
      override public string TranslateTypeName( string name )
      {
         if ( name == "Int16" )
            name = "Integer";
         else if ( name == "Int32" )
            name = "Long";
         else if ( name == "Int64" )
            name = "Long8";
         
         return name;
      }

      public string HTMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string XMLDeclaration
      {
         override get { throw new NotSupportedException( "Not yet implemented." ); }
      }

      public string Access
      {
         get
         {
            if ( m_TypeObject.IsPublic )
               return "Public ";
            else 
               return String.Empty;
         }
      }

      public string Attributes
      {
         get
         {
            StringBuilder s = new StringBuilder( m_ATTRIBUTE_STRING_SIZE );
         
            if ( m_TypeObject.IsAbstract && !m_TypeObject.IsInterface )
            {
               s.Append( "Abstract " );
            }
            else if ( m_TypeObject.IsSealed && !m_TypeObject.IsValueType && 
                      !m_TypeObject.IsSubclassOf( typeof( Delegate ) ) && 
                      !(m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               s.Append( "Sealed " );
            }
            return s.ToString();
         }
      }

      public string TypeCategory
      {
         get
         {
            string s = String.Empty;
            if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) &&
                 !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
            {
               StringBuilder sb = new StringBuilder( "Delegate " );
               MethodInfo m = m_TypeObject.GetMethod( "Invoke" );
               
               if ( m.ReturnType.Name.Equals( "Void" ) )
                  sb.Append( "Sub" );
               else
                  sb.Append( "Function" );

               sb.Append( " " );
               sb.Append( m_TypeObject.Name );

               ParameterInfo[] p = m.GetParameters();
               Type[] param_types = new Type[p.Length];
               for ( int i = 0; i < param_types.Length; i++ )
                  param_types[i] = p[i].ParameterType;

               sb.Append( "(" );
               for ( int i = 0; i < p.Length; i++ )
               {        
                  sb.Append( String.Format( "{0} {1}", TranslateTypeName( param_types[i].Name ), p[i].Name ) );
                  if ( i + 1 < p.Length )
                     sb.Append( ", " );
               }
            
               sb.Append( ")" );

               if ( !m.ReturnType.Name.Equals( "Void" ) )
               {                  
                  sb.Append( " As " );
                  sb.Append( TranslateTypeName( m.ReturnType.Name ) );
               }

               s = sb.ToString();
            }        
            else if ( m_TypeObject.IsInterface )
               s = "Interface";
            else if ( m_TypeObject.IsEnum )
               s = "Enum";
            else if ( m_TypeObject.IsValueType )
               s = "Struct";
            else
               s = "Class";
            return s;
         }
      }

      public string Extends
      {
         get
         {
             if (m_TypeObject.IsUnmanagedValueType)
               return String.Empty;

             if ( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) &&
                 !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) )
               return String.Empty;

            if ( m_TypeObject.IsSubclassOf( typeof( Enum ) ) && !( m_TypeObject == typeof( Enum ) ) )
               return String.Empty;
                           
            StringBuilder str = new StringBuilder();
                     
            bool bFound = false;

            // There is only a single parent class, if we are doing Object, then it has no parent,
            // otherwise, if the parent is just object, we can ignore that, it is assumed.
            if ( m_TypeObject != typeof( Object ) && 
                 m_TypeObject.BaseType != typeof( Object ) && 
                 !m_TypeObject.IsInterface )
            {
               bFound = true;
               str.Append( String.Format( " {0} {1}", "Inherits", m_TypeObject.BaseType.Name ) );
            }

            // Now we print the interfaces... there could be any number of these

            Hashtable hash = new Hashtable();
            for ( Type cur = m_TypeObject.BaseType; cur != null; cur = cur.BaseType )
            {               
               Type[] ifaces = cur.GetInterfaces();
               // Console.WriteLine( ifaces.Length );
               for ( int i = 0; i < ifaces.Length; i++ )
               {
                  // Console.WriteLine( ifaces[i].Name );
                  if ( !hash.ContainsKey( ifaces[i] ) )
                     hash.Add( ifaces[i], null );
               }
            }

            Type[] interfaces = m_TypeObject.GetInterfaces();
            ArrayList list = new ArrayList();
            // Console.WriteLine( interfaces.Length );
            for ( int i = 0; i < interfaces.Length; i++ )
            {
               if ( !hash.ContainsKey( interfaces[i] ) )
                  list.Add( interfaces[i] );
            }
                                   
            interfaces = new Type[list.Count];
		      for ( int listIndex = 0; listIndex < list.Count; listIndex++ )
			      interfaces[listIndex] = (Type)list[listIndex];

            if ( interfaces.Length != 0 )
            {
               string temp = interfaces[0].Name;
               if ( !bFound ) 
                  str.Append( " Implements " );
               else
                  str.Append( ", " );
               str.Append( temp );               
            }

            for ( int i = 1; i < interfaces.Length; i++ )
               str.Append( String.Format( ", {0}", interfaces[i].Name ) );

            return str.ToString();      
         }   
      }

      public string TextDeclaration
      {
         override get
         {
            String m_Extends = String.Empty;
            String m_TypeName = String.Empty;

            if ( !( m_TypeObject.IsSubclassOf( typeof( Delegate ) ) &&
                 !( m_TypeObject == typeof( Delegate ) || m_TypeObject == typeof( MulticastDelegate ) ) ) )
            {
               m_Extends = Extends;
               m_TypeName = " " + m_TypeObject.Name;
            }
            return Access + Attributes + TypeCategory + m_TypeName + m_Extends;
         }
      }

      public VisualBasicConstructorInfo[] OrderedLangConstructors
      {
         get
         {
            ConstructorInfo[] Constructors = null;
            VisualBasicConstructorInfo[] VisualBasicConstructors = null;
            try
            {
               Constructors = m_TypeObject.GetConstructors( BindingFlags.NonPublic );
               VisualBasicConstructors = new VisualBasicConstructorInfo[Constructors.Length];
               ConstructorComparer ICompConst = new ConstructorComparer();
               Array.Sort( Constructors, ICompConst );
               for ( int i = 0; i < Constructors.Length; i++ )
               {
                  VisualBasicConstructors[i] = new VisualBasicConstructorInfo( Constructors[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in VisualBasicLangType.OrderedConstructors.", Ex );
            }
            return VisualBasicConstructors;
         }
      }

      public VisualBasicMethodInfo[] OrderedLangMethods
      {
         get
         {
            MethodInfo[] Methods = null;
            VisualBasicMethodInfo[] VisualBasicMethods = null;
            try
            {
               Methods = m_TypeObject.GetMethods( BindingFlags.NonPublic );
               VisualBasicMethods = new VisualBasicMethodInfo[Methods.Length];
               
               // Order the member categories
               MethodComparer ICompMet = new MethodComparer();
               Array.Sort( Methods, ICompMet );
               for ( int i = 0; i < Methods.Length; i++ )               
                  VisualBasicMethods[i] = new VisualBasicMethodInfo( Methods[i] );               
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in VisualBasicLangType.OrderedConstructors.", Ex );
            }
            return VisualBasicMethods;
         }
      }

      public VisualBasicPropertyInfo[] OrderedLangProperties
      {
         get
         {
            int i = 0, j = 0;
            PropertyInfo[] Properties = null;
            VisualBasicPropertyInfo[] VisualBasicProperties = null;
            try
            {
               Properties = m_TypeObject.GetProperties( BindingFlags.NonPublic );
               VisualBasicProperties = new VisualBasicPropertyInfo[Properties.Length];
               PropertyComparer ICompProp = new PropertyComparer();
               Array.Sort( Properties, ICompProp );
               for ( ; i < Properties.Length; i++ )
               {
                  try
                  {
                     VisualBasicProperties[j] = new VisualBasicPropertyInfo( Properties[i] );
                     j++;
                  }                  
                  catch ( Exception /*ex*/ )
                  {
                     // Console.WriteLine( "VisualBasicProperty error {0}.{1}", Properties[i].ReflectedType.FullName, Properties[i].Name );
                  }                     
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in VisualBasicLangType.OrderedProperties", Ex );
            }

            Console.WriteLine( "VB" );

            VisualBasicPropertyInfo[] np = new VisualBasicPropertyInfo[j];
            for ( i = 0; i < j; i++ )
               np[i] = VisualBasicProperties[i];
            return np;
         }
      }
            
      public VisualBasicFieldInfo[] OrderedLangFields
      {
         get
         {
            FieldInfo[] Fields = null;
            VisualBasicFieldInfo[] VisualBasicFields = null;
            try
            {
               Fields = m_TypeObject.GetFields( BindingFlags.NonPublic );
               VisualBasicFields = new VisualBasicFieldInfo[Fields.Length];
               FieldComparer ICompField = new FieldComparer();
               Array.Sort( Fields, ICompField );
               for ( int i = 0; i < Fields.Length; i++ )
               {
                  VisualBasicFields[i] = new VisualBasicFieldInfo( Fields[i] );
               }
            }
            catch ( Exception Ex )
            {
               throw new Exception( "Occurred in VisualBasicLangType.OrderedFields.", Ex );
            }
            return VisualBasicFields;
         }
      }
   }

} // namespace LangUtil
